#include <QtGui>

#include "treeitem.h"
#include "treemodel.h"

QModelIndex TreeModel::ixCur=QModelIndex();
TreeItem* TreeModel::itStartParent=NULL;

TreeModel::TreeModel(QObject *parent)
    : QAbstractItemModel(parent)
{
    QList<QVariant> rootData;
    rootData << "Heading" << "Type" <<"Note";
    rootItem = new TreeItem(rootData);
}

TreeModel::~TreeModel()
{
    delete rootItem;
}

int TreeModel::columnCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return static_cast<TreeItem*>(parent.internalPointer())->columnCount();
    else
        return rootItem->columnCount();
}

QVariant TreeModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole)
        return QVariant();

    TreeItem *item = static_cast<TreeItem*>(index.internalPointer());

    return item->data(index.column());
}

Qt::ItemFlags TreeModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return Qt::ItemIsEnabled;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant TreeModel::headerData(int section, Qt::Orientation orientation,
                               int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
        return rootItem->data(section);

    return QVariant();
}

QModelIndex TreeModel::index(int row, int column, const QModelIndex &parent)
            const
{
    TreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<TreeItem*>(parent.internalPointer());

    TreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex TreeModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    TreeItem *childItem = static_cast<TreeItem*>(index.internalPointer());
    TreeItem *parentItem = childItem->parent();

    if (parentItem == rootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

int TreeModel::rowCount(const QModelIndex &parent) const
{
    TreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<TreeItem*>(parent.internalPointer());

    return parentItem->childCount();
}

TreeItem* TreeModel::first()
{
	if (rootItem->childCount()  ==0)
	{
		ixCur=QModelIndex();
		return NULL;
	} 
	ixCur=index (0,0);
	return 	static_cast<TreeItem*>(ixCur.internalPointer());
}
	
TreeItem* TreeModel::next()
{
	if (!ixCur.isValid() )
		return NULL;

	ixCur=index (ixCur.row()+1,0);
	if (ixCur.isValid())
		return 	static_cast<TreeItem*>(ixCur.internalPointer());
	return NULL;	

}

#include <iostream>
bool TreeModel::removeRows ( int row, int count, const QModelIndex & parent)
{
	int last=row+count-1;
	beginRemoveRows (parent,row,last);

    TreeItem *pi= static_cast<TreeItem*>(parent.internalPointer());
	for (int i=row; i<=last; i++)
	{
		std::cout << "TreeModel::removeRows removing i="<<i<<std::endl;
		pi->removeChild (row);
	}

	endRemoveRows ();
	
}

QModelIndex TreeModel::index (TreeItem* ti)
{
	return createIndex (ti->row(),ti->column(),ti);
}

